// Copyright 2012-2014, Philip Quinn. All Rights Reserved.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//
//  TSSEventRecorder.m
//  TouchScrollSimulator
//
//  Created by Philip Quinn on 20/04/12.
//  Copyright (c) 2012 __MyCompanyName__. All rights reserved.
//

#import "TSEventRecorder.h"
#import "PrivateAPI/GSEvent.h"

@interface TSEventRecorder ()
-(void)writeToFile:(NSString*)path;
@end

@implementation TSEventRecorder
@synthesize events = _events;

+(NSArray*)findRecordedScripts
{
    // Files in ~/Documents
    NSArray *dirs = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
    NSString *docDir = [dirs lastObject];
    NSArray *files = [[NSFileManager defaultManager] contentsOfDirectoryAtPath:docDir
                                                                         error:nil];
    
    // Find all *.plist files
    NSMutableArray *scripts = [[NSMutableArray alloc] init];
    [files enumerateObjectsUsingBlock:^(id obj, NSUInteger idx, BOOL *stop) {
        if (![[obj pathExtension] isEqualToString:@"plist"])
            return;
        
        NSURL *url = [NSURL fileURLWithPath:[docDir stringByAppendingPathComponent:obj]];
        [scripts addObject:url];
    }];
    return scripts;
}

-(id)init 
{
    if ((self = [super init])) {
        _events = [[NSMutableArray alloc] init];
    }
    return self;
}

-(id)initWithContentsOfURL:(NSURL*)url
{
    if ((self = [super init])) {
        _events = [[NSMutableArray alloc] initWithContentsOfURL:url];
    }
    return self;
}

-(void)recordApplicationEvent:(NSDictionary*)event 
{
    // The dictionary doesn't include details about the type of touch event
    // (touch down/touch moved/touch up/etc.) Record this information in the
    // dictionary so other apps can read it too.
    GSEventRef gse = GSEventCreateWithPlist((__bridge CFDictionaryRef)(event));
    /*
    if (GSEventGetType(gse) == kGSEventHand) {
        NSMutableDictionary *suppliment = [event mutableCopy];
        GSHandInfo hand = GSEventGetHandInfo(gse);
        if (hand.type == kGSHandInfoTypeTouchDown)
            [suppliment setObject:@"kGSHandInfoTypeTouchDown" forKey:@"GSHandInfoType"];
        else if (hand.type == kGSHandInfoTypeTouchDragged)
            [suppliment setObject:@"kGSHandInfoTypeTouchDragged" forKey:@"GSHandInfoType"];
        else if (hand.type == kGSHandInfoTypeTouchMoved)
            [suppliment setObject:@"kGSHandInfoTypeTouchMoved" forKey:@"GSHandInfoType"];
        else if (hand.type == kGSHandInfoTypeTouchUp)
            [suppliment setObject:@"kGSHandInfoTypeTouchUp" forKey:@"GSHandInfoType"];
        else if (hand.type == kGSHandInfoTypeCancel)
            [suppliment setObject:@"kGSHandInfoTypeCancel" forKey:@"GSHandInfoType"];
        
        event = suppliment;
    }
    */
    NSMutableDictionary *suppliment = [event mutableCopy];
    if (GSEventGetType(gse) == kGSEventLeftMouseDown)
        [suppliment setObject:@"kGSEventLeftMouseDown" forKey:@"GSEventType"];
    else if (GSEventGetType(gse) == kGSEventLeftMouseUp)
        [suppliment setObject:@"kGSEventLeftMouseUp" forKey:@"GSEventType"];
    else if (GSEventGetType(gse) == kGSEventLeftMouseDragged)
        [suppliment setObject:@"kGSEventLeftMouseDragged" forKey:@"GSEventType"];
    
    [suppliment setObject:[NSNumber numberWithDouble:GSEventGetTimestamp(gse)]
                   forKey:@"CFAbsoluteTime"];
    
    event = suppliment;
    
    CFRelease(gse);
    [_events addObject:event];
}

-(void)removeLastEvent
{
    [_events removeLastObject];
}

-(void)writeWithLabel:(NSString*)label
{
    NSArray *dirs = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
    NSString *docDir = [dirs lastObject];
    
    NSString *path = [docDir stringByAppendingPathComponent:[NSString stringWithFormat:@"%@.plist", label]];
    int extender = 1;
    while ([[NSFileManager defaultManager] fileExistsAtPath:path]) {
        path = [docDir stringByAppendingPathComponent:[NSString stringWithFormat:@"%@ %d.plist", label, extender++]];
    }

    [self writeToFile:path];
}

-(void)writeToURL:(NSURL*)url 
{
    [_events writeToURL:url atomically:YES];
}

-(void)writeToFile:(NSString*)path 
{
    [_events writeToFile:path atomically:YES];
}

@end
