// Copyright 2012-2014, Philip Quinn. All Rights Reserved.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//
//  TSRecordedScriptsController.m
//  TouchScrollSimulator
//

#import "TSRecordedScriptsController.h"
#import "TSEventRecorder.h"

static NSString *kScriptCellIdentifier = @"ScriptCell";

// Keys in recordedScripts dictionaries
static NSString *kScriptNameKey = @"Name";
static NSString *kScriptPathKey = @"Path";
static NSString *kScriptDateKey = @"Date";

@interface TSRecordedScriptsController ()
// Array of dictionaries
@property (strong) NSArray *recordedScripts;
// Cache a date formatter for cell rendering
@property (strong) NSDateFormatter *dateFormatter;
@end

@implementation TSRecordedScriptsController
@synthesize delegate = _delegate;
@synthesize recordedScripts = _recordedScripts;
@synthesize dateFormatter = _dateFormatter;

-(void)viewDidLoad
{
    [super viewDidLoad];
    // Cache date formatter
    self.dateFormatter = [[NSDateFormatter alloc] init];
    [self.dateFormatter setTimeStyle:NSDateFormatterShortStyle];
    [self.dateFormatter setDateStyle:NSDateFormatterLongStyle];

    // Fetch name/path/date info about script files
    NSMutableArray *scripts = [[NSMutableArray alloc] init];
    NSArray *scriptURLs = [TSEventRecorder findRecordedScripts];    
    [scriptURLs enumerateObjectsUsingBlock:^(id obj, NSUInteger idx, BOOL *stop) {
        NSURL *url = obj;
        NSString *name = [[url lastPathComponent] stringByDeletingPathExtension];
        NSDate *lastModified = nil;
        [url getResourceValue:&lastModified forKey:NSURLContentModificationDateKey error:nil];
        
        [scripts addObject:[NSDictionary dictionaryWithObjectsAndKeys:
                                name, kScriptNameKey,
                                url, kScriptPathKey,
                                lastModified, kScriptDateKey, nil]];
    }];
    self.recordedScripts = scripts;
}

-(IBAction)closeWithoutSelection:(id)sender
{
    [self.delegate selectedRecordedScriptURL:nil];
    [self dismissModalViewControllerAnimated:YES];
}

#pragma mark UITableViewDataSource

-(NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    return [self.recordedScripts count];
}

-(UITableViewCell*)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:kScriptCellIdentifier];
    if (!cell) {
        cell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleSubtitle 
                                      reuseIdentifier:kScriptCellIdentifier];
    }
    
    NSDictionary *data = [self.recordedScripts objectAtIndex:indexPath.row];
    cell.textLabel.text = [data objectForKey:kScriptNameKey];
    cell.detailTextLabel.text = [self.dateFormatter stringFromDate:[data objectForKey:kScriptDateKey]];
    return cell;
}

#pragma mark UITableViewDelegate

-(void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    // Pass URL back to delegate
    NSURL *url = [[self.recordedScripts objectAtIndex:indexPath.row] objectForKey:kScriptPathKey];
    [self.delegate selectedRecordedScriptURL:url];
    [self dismissModalViewControllerAnimated:YES];
}

@end
