// Copyright 2012-2014, Philip Quinn and Sylvain Malacria. All Rights Reserved.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//
//  TSStripedView.m
//  TouchScrollSimulator
//

#import "TSStripedView.h"
#import "TSStripedLayer.h"

@interface TSStripedView ()
-(void)_init;
@end

@implementation TSStripedView
+ (Class)layerClass 
{
    return [TSStripedLayer class];
}

-(void)_init
{
    self.userInteractionEnabled = NO;
    self.primaryColor = [UIColor darkGrayColor];
    self.secondaryColor = [UIColor grayColor];
    self.stripeHeight = 50.f;
    self.drawingEnabled = YES;
    
    [self setFrame:CGRectMake([self frame].origin.x, [self frame].origin.y, [self frame].size.width, 1000000.0f)];
}

-(id)initWithFrame:(CGRect)frame
{
    if ((self = [super initWithFrame:frame])) {
        [self _init];
    }
    return self;
}

-(id)initWithCoder:(NSCoder *)aDecoder
{
    if ((self = [super initWithCoder:aDecoder])) {
        [self _init];
    }
    return self;
}

-(UIColor*)primaryColor { return _primaryColor; }
-(void)setPrimaryColor:(UIColor *)primaryColor
{
    _primaryColor = primaryColor;
    [self setNeedsDisplay];
}

-(UIColor*)secondaryColor { return _secondaryColor; }
-(void)setSecondaryColor:(UIColor *)secondaryColor
{
    _secondaryColor = secondaryColor;
    [self setNeedsDisplay];
}

-(CGFloat)stripeHeight { return _stripeHeight; }
-(void)setStripeHeight:(CGFloat)stripeHeight
{
    _stripeHeight = stripeHeight;
    [self setNeedsDisplay];
}

-(BOOL)isDrawingEnabled { return _drawingEnabled; }
-(void)setDrawingEnabled:(BOOL)drawingEnabled
{
    _drawingEnabled = drawingEnabled;
    [self setNeedsDisplay];
}

-(void)drawRect:(CGRect)rect
{
    if (!self.drawingEnabled) {
        return;
    }
    
    CGContextRef context = UIGraphicsGetCurrentContext();
    CGFloat nextY = CGRectGetMinY(rect);
    UIColor *colour = nil;
       
    // Is the top-left point in a black or white stripe?
    if ((int)(nextY/self.stripeHeight)%2 == 0) {
        colour = self.primaryColor;
    } else {
        colour = self.secondaryColor;
    }
    
    while (nextY <= CGRectGetMaxY(rect)) {
        CGRect stripeRect = CGRectMake(CGRectGetMinX(rect), nextY, 
                                       CGRectGetWidth(rect), self.stripeHeight - ((int)nextY)%(int)self.stripeHeight);
        CGContextSetFillColorWithColor(context, colour.CGColor);
        CGContextFillRect(context, stripeRect);
        
        nextY = CGRectGetMaxY(stripeRect);        
        colour = [colour isEqual:self.primaryColor] ? self.secondaryColor : self.primaryColor;
    }
}

@end
